#ifndef NODELIB_INCLUDE

#define NODELIB_INCLUDE 1


typedef struct Node {
                          struct Node *next;
                          struct Node *prev;
                      } Node;


typedef struct NodeList {
                             Node *first;  /* pointer to first node*/
                             Node *last;   /* pointer to last node*/
                             int  count;   /* nr of items*/
                          } NodeList;

typedef void(NodeFreeFunc)(void*);
typedef int(*NodeCompareFunc)(Node *,Node*);

/* This routine should allways be called before actually doing something with the list  */
void InitNodeList(NodeList *l);        

/* This routine initializes one node */
void InitNode(Node *n);

/*
 *   Add a node at the front and a node at the back
 */
void AddBackNode(NodeList *list,Node *n);
void AddFrontNode(NodeList *list,Node *n);

/*
 *  Insert node n1  directly after node n2
 */
void InsertNode(NodeList *l,Node *n1,Node *n2);

/*
 *  InsertSorted Node >> use the compare function to compare nodes and insert the node
 *  at it's ordinal position.
 *
 *  the compare function is a strcmp like function that has 2 nodes as input and returns
 *  -1 on n1 < n2
 *   0 on n1 = n2
 *   1 on n1 > n2
 *
 *  example:
 *
 *   int MyNodeCompareFunc(MyNodeType *n1,MyNodeType *n2)
 *   {
 *       if (n1->MyInteger==n2->MyInteger)
 *       {
 *           return(0);
 *       }
 *       else if (n1->MyInteger < n2->MyInteger)
 *       {
 *           return(-1);
 *       }
 *       else
 *       {
 *           return(1);
 *       }
 *   }
 *
 *
 *  usage : InsertSortedNode((NodeList *)MyListType,(Node *)node,(NodeCompareFunc)MyCompareFunc);
 */
void InsertSortedNode(NodeList *l,Node *n1,NodeCompareFunc CompareNode);

/*
 *  Detach >> cut a node loose from the list (do not delete it)
 *
 *  example : get the second node from a list and detach it to place it into another list
 *
 *    n=FirstNode((NodeList *) list1);
 *    n = NextNode(n);               
 *    DetachNode((NodeList *) list1,n);  
 *    AddBackNode((NodeList *)list2,n);
 */
Node *DetachNode(NodeList *l,Node *n);

/*
 *  this function wipes a complete nodelist from memory, the delete function should have a node-pointer
 *  as parameter : 
 *
 * assume we have the following node   struct MyNode {
 *                                                       Node __node;
 *                                                       char *str;
 *                                                   } MyNode
 * The string in the node is dynamically allocated, this means that if the node is deleted the str also
 * needs to be freed, the free function should look something like this :
 *
 *  void MyNodeFree(MyNode *n)
 *  {
 *      if (n)
 *      {
 *          if (n->str)
 *          {
 *              free(n->str);
 *          }
 *          free(n);
 *      }
 *  }
 */
void DeleteNodeList(NodeList *l,NodeFreeFunc DeleteNode);

/*
 * the FirstNode, LastNode, NextNode and PrevNode routines are used to step through a node list : 
 * example : forward :
 *    n=FirstNode((NodeList *) list);
 *    while (n)
 *    {
 *        n=NextNode(n);
 *    }
 *
 * backwards : 
 *    n = LastNode((NodeList *)l);
 *    while (n)
 *    {
 *        n = PrevNode(n);
 *    }
 */
Node *FirstNode(NodeList *l);
Node *LastNode(NodeList *l);
Node *NextNode(Node *i);
Node *PrevNode(Node *i);

/*
 *  these are the simpelest delete functions for a node (remember, the memory is not freed !!)) 
 */
Node *DeleteFirst(NodeList *l);
Node *DeleteLast(NodeList *l);



/**/
/* NodeCount(NodeList *l)  >> returns the number of nodes in a NodeList*/
/**/
int NodeCount(NodeList *l);


/**********************************************************************************************************
 *
 * Complete programming example :
 *
 * typedef struct MyNode {
 *                          Node *_InternallyUsedNode;   >>> this makes it possible to use the Node Functions                       
 *                          char *str;
 *                      } MyNode;
 *
 * typedef struct MyNodeList { 
 *                             NodeList _InternalNodeList; >>> this makes it possible to use the node functions 
 *                             int type_of_content;
 *                             int otherdatathatcomeswiththelist;
 *                           } MyNodeList;
 *                           
 * MyNode *NewNode(char *str)
 * {
 *     MyNode *res;
 *     
 *     res = alloc(sizeof(MyNode));
 *     if (res)
 *     {
 *         res->str = alloc(strlen(str)+4);
 *         if (res->str)
 *         {
 *             strcpy(res->str,str);
 *         } 
 *         else
 *         {
 *             free(res);
 *             res = NULL;
 *         }
 *     }
 * }
 *
 * void DeleteNode(MyNode *n)
 * {
 *     if (n)
 *     {
 *         if (n->str)
 *         {
 *             free(n->str);
 *         }
 *         free(n);
 *     }
 * }
 * 
 * void main(void)
 * {
 *     MyNode *n;
 *     MyNodeList list;
 *    
 *     InitNodeList((NodeList *)&list);
 *    
 *     n = NewNode("First String");
 *       >>> a check if allocation succeeded (n!=NULL) should follow but for the sake of the example we don't at this moment
 *     AddBackNode((NodeList *)&list,n);
 *    
 *     n = NewNode("Second String");
 *     AddBackNode((NodeList *)&list,n);
 *    
 *     n = NewNode("Last String");
 *     AddBackNode((NodeList *)&list,n);
 *    
 *     >>> go through list forward 
 *     n = FirstNode((NodeList *)&list);
 *     while (n)
 *     {
 *         printf(" >> %s\n",n->str);
 *         n = NextNode((Node *)n);
 *     }
 *
 *     >>> go through list Backwards 
 *     n = LastNode((NodeList *)&list);
 *     while (n)
 *     {
 *         printf(" >> %s\n",n->str);
 *         n = PrevNode((Node *)n);
 *     }
 *     
 *     >>> Delete the complete list using our custom 'nodedelete function' 
 *     DeleteNodeList((NodeList *)&list,(NodeFreeFunc)DeleteNode);
 *     
 *     >>> all mem is freed by deleting the list and we are done 
 * }
 */
#endif
